//******************************************************************************
//! @file       cc1120_rx_sniff_mode_rx.c
//! @brief      This program demonstrates Rx Sniff Mode for the 1.2 kbps,
//              50 kHz RX filter BW setting available in SmartRf Studio.
//              The transmitter sends a packet every time a button is pushed and
//              the receiver implements RX Sniff Mode to reduce the current
//              consumption.
//              DN511 (http://www.ti.com/lit/swra428) explains how the register
//              settings are found.
//
//  Copyright (C) 2013 Texas Instruments Incorporated - http://www.ti.com/
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions
//  are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of Texas Instruments Incorporated nor the names of
//      its contributors may be used to endorse or promote products derived
//      from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//*****************************************************************************/


/*******************************************************************************
* INCLUDES
*/
#include "msp430.h"
#include "lcd_dogm128_6.h"
#include "hal_spi_rf_trxeb.h"
#include "cc112x_spi.h"
#include "stdlib.h"
#include "bsp.h"
#include "bsp_key.h"
#include "io_pin_int.h"
#include "bsp_led.h"
#include "cc1120_rx_sniff_mode_reg_config.h"


/*******************************************************************************
* DEFINES
*/
#define ISR_ACTION_REQUIRED     1
#define ISR_IDLE                0

#define GPIO3                   0x04
#define GPIO2                   0x08
#define GPIO0                   0x80

#define WMBUS2400               1
#define WMBUS4800               2
/*******************************************************************************
* LOCAL VARIABLES
*/
static uint8  packetSemaphore;
static uint32 packetCounter = 0;
static uint8  wmbusMode;
static uint8  buttonPressed = 0x00;

/*******************************************************************************
* STATIC FUNCTIONS
*/
static void initMCU(void);
static void registerConfig(uint8 config);
static void manualCalibration(void);
static void runRX(void);
static void radioRxISR(void);
static void updateLcd(void);
static void calibrateRCOsc(void);
static void welcomeScreen(void);


/*******************************************************************************
*   @fn         main
*
*   @brief      Runs the main routine
*
*   @param      none
*
*   @return     none
*/
void main(void) {
  
  // Initialize MCU and peripherals
  initMCU();
  
  // Put radio to sleep
  trxSpiCmdStrobe(CC112X_SPWD);
  
  welcomeScreen();
  
  // Wait for button push
  while(!buttonPressed){
    buttonPressed = bspKeyPushed(BSP_KEY_ALL);
  }
  if(buttonPressed == BSP_KEY_RIGHT){
    wmbusMode = WMBUS4800;
  }
  else{
    wmbusMode = WMBUS2400;
  }
  buttonPressed = 0;
  
  // Write radio registers
  registerConfig(wmbusMode);
  
  // Enter runRX, never coming back
  runRX();
}


/*******************************************************************************
*   @fn         runRX
*
*   @brief      Puts radio in RX Sniff Mode and waits for packets. A packet
*               counter is incremented for each packet received and the LCD is
*               updated
*
*   @param      none
*
*   @return     none
*/
static void runRX(void) {
  
  uint8 rxBuffer[128] = {0};
  uint8 rxBytes;
  uint8 marcState;
  
  // Connect ISR function to GPIO2
  ioPinIntRegister(IO_PIN_PORT_1, GPIO2, &radioRxISR);
  
  // Interrupt on falling edge
  ioPinIntTypeSet(IO_PIN_PORT_1, GPIO2, IO_PIN_RISING_EDGE);
  
  // Clear ISR flag
  ioPinIntClear(IO_PIN_PORT_1, GPIO2);
  
  // Enable interrupt
  ioPinIntEnable(IO_PIN_PORT_1, GPIO2);
  
  // Update LCD
  updateLcd();
  
  // Calibrate radio according to errata
  manualCalibration();
  
  // Calibrate the RCOSC
  calibrateRCOsc();
  
  // Infinite loop
  while(TRUE) {
    
    // Set radio in RX Sniff Mode
    trxSpiCmdStrobe(CC112X_SWOR);
    
    // Wait for packet to be received
    while(packetSemaphore != ISR_ACTION_REQUIRED);
    
    // Clear semaphore flag
    packetSemaphore = ISR_IDLE;
    
    // Read number of bytes in RX FIFO
    cc112xSpiReadReg(CC112X_NUM_RXBYTES, &rxBytes, 1);
    
    // Read all the bytes in the RX FIFO
    cc112xSpiReadRxFifo(rxBuffer, rxBytes);
    
    // Update LCD
    updateLcd();
  }
}


/*******************************************************************************
*   @fn         calibrateRcOsc
*
*   @brief      Calibrates the RC oscillator used for the eWOR timer. When this
*               function is called, WOR_CFG0.RC_PD must be 0
*
*   @param      none
*
*   @return     none
*/
static void calibrateRCOsc(void) {
  
  uint8 temp;
  
  // Read current register value
  cc112xSpiReadReg(CC112X_WOR_CFG0, &temp,1);
  
  // Mask register bit fields and write new values
  temp = (temp & 0xF9) | (0x02 << 1);
  
  // Write new register value
  cc112xSpiWriteReg(CC112X_WOR_CFG0, &temp,1);
  
  // Strobe IDLE to calibrate the RCOSC
  trxSpiCmdStrobe(CC112X_SIDLE);
  
  // Disable RC calibration
  temp = (temp & 0xF9) | (0x00 << 1);
  cc112xSpiWriteReg(CC112X_WOR_CFG0, &temp, 1);
}


/*******************************************************************************
*   @fn         radioRxISR
*
*   @brief      ISR for packet handling in RX. Sets packet semaphore
*               and clears interrupt flag
*
*   @param      none
*
*   @return     none
*/
static void radioRxISR(void) {
  
  // Set packet semaphore
  packetSemaphore = ISR_ACTION_REQUIRED;
  
  // Clear ISR flag
  ioPinIntClear(IO_PIN_PORT_1, GPIO2);
}


/*******************************************************************************
*   @fn         initMCU
*
*   @brief      Initialize MCU and board peripherals
*
*   @param      none
*
*   @return     none
*/
static void initMCU(void) {
  
  // Init clocks and I/O
  bspInit(BSP_SYS_CLK_8MHZ);
  
  // Init LEDs
  bspLedInit();
  
  // Init buttons
  bspKeyInit(BSP_KEY_MODE_POLL);
  
  // Initialize SPI interface to LCD (shared with SPI flash)
  bspIoSpiInit(BSP_FLASH_LCD_SPI, BSP_FLASH_LCD_SPI_SPD);
  
  // Init LCD
  lcdInit();
  
  // Instantiate transceiver RF SPI interface to SCLK ~ 4 MHz
  // Input parameter is clockDivider
  // SCLK frequency = SMCLK/clockDivider
  trxRfSpiInterfaceInit(2);
  
  // Enable global interrupt
  _BIS_SR(GIE);
}


/*******************************************************************************
*   @fn         registerConfig
*
*   @brief      Write register settings as given by SmartRF Studio found in
*               cc1120_rx_sniff_mode_reg_config.h
*
*   @param      none
*
*   @return     none
*/
static void registerConfig(uint8 config) {
  
  uint8 writeByte;
  
  // Reset radio
  trxSpiCmdStrobe(CC112X_SRES);
  
  if( config == WMBUS2400){
    // Write registers to radio
    for(uint16 i = 0;
        i < (sizeof(preferredSettings2400)/sizeof(registerSetting_t)); i++) {
          writeByte = preferredSettings2400[i].data;
          cc112xSpiWriteReg(preferredSettings2400[i].addr, &writeByte, 1);
        }
  }
  else{
    // Write registers to radio
    for(uint16 i = 0;
        i < (sizeof(preferredSettings4800)/sizeof(registerSetting_t)); i++) {
          writeByte = preferredSettings4800[i].data;
          cc112xSpiWriteReg(preferredSettings4800[i].addr, &writeByte, 1);
        }
    
  }
}



/*******************************************************************************
*   @fn         updateLcd
*
*   @brief      Updates LCD buffer and sends buffer to LCD module
*
*   @param      none
*
*   @return     none
*/
static void updateLcd(void) {
  
  // Update LDC buffer and send to screen
  // Update LDC buffer and send to screen
  lcdBufferClear(0);
  lcdBufferPrintString(0, "RX Sniff Mode", 0, eLcdPage0);
  lcdBufferSetHLine(0, 0, LCD_COLS - 1, 7);
  lcdBufferPrintString(0, "Received OK:", 0, eLcdPage3);
  lcdBufferPrintInt(0, packetCounter++, 70, eLcdPage4);
  if( wmbusMode == WMBUS2400){
    lcdBufferPrintString(0, "RX, WMBUS 2400", 0, eLcdPage7);
  }
  else if( wmbusMode == WMBUS4800){
    lcdBufferPrintString(0, "RX, WMBUS 4800", 0, eLcdPage7);
  }
  lcdBufferSetHLine(0, 0, LCD_COLS - 1, 55);
  lcdBufferInvertPage(0, 0, LCD_COLS, eLcdPage7);
  lcdSendBuffer(0);
}
/*******************************************************************************
*   @fn         welcomeScreen
*
*   @brief      Updates LCD buffer and sends buffer to LCD module
*
*   @param      none
*
*   @return     none
*/
static void welcomeScreen(void) {
  
  // Update LDC buffer and send to screen
  lcdBufferClear(0);
  lcdBufferPrintString(0, "RX Sniff Mode", 0, eLcdPage0);
  lcdBufferSetHLine(0, 0, LCD_COLS - 1, 7);
  lcdBufferPrintString(0, "Choose mode:", 0, eLcdPage3);
  lcdBufferPrintString(0, "Left:  WMBUS 2400", 0, eLcdPage4);
  lcdBufferPrintString(0, "Right: WMBUS 4800", 0, eLcdPage5);
  lcdBufferPrintString(0, "RX", 10, eLcdPage7);  
  lcdBufferSetHLine(0, 0, LCD_COLS - 1, 55);
  lcdBufferInvertPage(0, 0, LCD_COLS, eLcdPage7);
  lcdSendBuffer(0);
}  

/*******************************************************************************
*   @fn         manualCalibration
*
*   @brief      Calibrates radio according to CC112x errata
*
*   @param      none
*
*   @return     none
*/
#define VCDAC_START_OFFSET 2
#define FS_VCO2_INDEX 0
#define FS_VCO4_INDEX 1
#define FS_CHP_INDEX 2
static void manualCalibration(void) {
  
  uint8 original_fs_cal2;
  uint8 calResults_for_vcdac_start_high[3];
  uint8 calResults_for_vcdac_start_mid[3];
  uint8 marcstate;
  uint8 writeByte;
  
  // 1) Set VCO cap-array to 0 (FS_VCO2 = 0x00)
  writeByte = 0x00;
  cc112xSpiWriteReg(CC112X_FS_VCO2, &writeByte, 1);
  
  // 2) Start with high VCDAC (original VCDAC_START + 2):
  cc112xSpiReadReg(CC112X_FS_CAL2, &original_fs_cal2, 1);
  writeByte = original_fs_cal2 + VCDAC_START_OFFSET;
  cc112xSpiWriteReg(CC112X_FS_CAL2, &writeByte, 1);
  
  // 3) Calibrate and wait for calibration to be done
  //   (radio back in IDLE state)
  trxSpiCmdStrobe(CC112X_SCAL);
  
  do {
    cc112xSpiReadReg(CC112X_MARCSTATE, &marcstate, 1);
  } while (marcstate != 0x41);
  
  // 4) Read FS_VCO2, FS_VCO4 and FS_CHP register obtained with 
  //    high VCDAC_START value
  cc112xSpiReadReg(CC112X_FS_VCO2,
                   &calResults_for_vcdac_start_high[FS_VCO2_INDEX], 1);
  cc112xSpiReadReg(CC112X_FS_VCO4,
                   &calResults_for_vcdac_start_high[FS_VCO4_INDEX], 1);
  cc112xSpiReadReg(CC112X_FS_CHP,
                   &calResults_for_vcdac_start_high[FS_CHP_INDEX], 1);
  
  // 5) Set VCO cap-array to 0 (FS_VCO2 = 0x00)
  writeByte = 0x00;
  cc112xSpiWriteReg(CC112X_FS_VCO2, &writeByte, 1);
  
  // 6) Continue with mid VCDAC (original VCDAC_START):
  writeByte = original_fs_cal2;
  cc112xSpiWriteReg(CC112X_FS_CAL2, &writeByte, 1);
  
  // 7) Calibrate and wait for calibration to be done
  //   (radio back in IDLE state)
  trxSpiCmdStrobe(CC112X_SCAL);
  
  do {
    cc112xSpiReadReg(CC112X_MARCSTATE, &marcstate, 1);
  } while (marcstate != 0x41);
  
  // 8) Read FS_VCO2, FS_VCO4 and FS_CHP register obtained 
  //    with mid VCDAC_START value
  cc112xSpiReadReg(CC112X_FS_VCO2, 
                   &calResults_for_vcdac_start_mid[FS_VCO2_INDEX], 1);
  cc112xSpiReadReg(CC112X_FS_VCO4,
                   &calResults_for_vcdac_start_mid[FS_VCO4_INDEX], 1);
  cc112xSpiReadReg(CC112X_FS_CHP,
                   &calResults_for_vcdac_start_mid[FS_CHP_INDEX], 1);
  
  // 9) Write back highest FS_VCO2 and corresponding FS_VCO
  //    and FS_CHP result
  if (calResults_for_vcdac_start_high[FS_VCO2_INDEX] >
      calResults_for_vcdac_start_mid[FS_VCO2_INDEX]) {
        writeByte = calResults_for_vcdac_start_high[FS_VCO2_INDEX];
        cc112xSpiWriteReg(CC112X_FS_VCO2, &writeByte, 1);
        writeByte = calResults_for_vcdac_start_high[FS_VCO4_INDEX];
        cc112xSpiWriteReg(CC112X_FS_VCO4, &writeByte, 1);
        writeByte = calResults_for_vcdac_start_high[FS_CHP_INDEX];
        cc112xSpiWriteReg(CC112X_FS_CHP, &writeByte, 1);
      } else {
        writeByte = calResults_for_vcdac_start_mid[FS_VCO2_INDEX];
        cc112xSpiWriteReg(CC112X_FS_VCO2, &writeByte, 1);
        writeByte = calResults_for_vcdac_start_mid[FS_VCO4_INDEX];
        cc112xSpiWriteReg(CC112X_FS_VCO4, &writeByte, 1);
        writeByte = calResults_for_vcdac_start_mid[FS_CHP_INDEX];
        cc112xSpiWriteReg(CC112X_FS_CHP, &writeByte, 1);
      }
}